package com.xy.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import com.xy.config.SaTokenAuthorizeConfig;
import com.xy.dto.AuthorizeDto;
import com.xy.dto.SysMenuDto;
import com.xy.redis.RedisCache;
import com.xy.service.AuthorizeService;
import com.xy.utils.R;
import com.xy.utils.SaTokenUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Api(tags = "鉴权相关接口")
@AllArgsConstructor
public class AuthorizeServiceImpl implements AuthorizeService {

    private SysMenuServiceImpl sysMenuService;

    private SaTokenAuthorizeConfig saTokenAuthorizeConfig;

    @Override
    @ApiOperation("获取菜单")
    public R<List<SysMenuDto.Vo>> workMenu(SysMenuDto.WorkMenu workMenu) {
        return sysMenuService.workMenu(workMenu);
    }

    @Override
    @ApiOperation("判断数据查询权限")
    public R<Boolean> getDataAuth(AuthorizeDto authorizeDto) {
        boolean dataAuth = RedisCache.getDataAuth(SaTokenUtils.getToken(), authorizeDto.getSystemId(), authorizeDto.getUri());
        return R.ok(dataAuth);
    }

    @Override
    @ApiOperation("获取所有系统ID")
    public R<List<Long>> getSystemIds() {
        List<Long> systemIds = RedisCache.getSystemIds(SaTokenUtils.getToken());
        return R.ok(systemIds);
    }

    @Override
    @ApiOperation("获取所有角色ID")
    public R<List<Long>> getRoleIds(AuthorizeDto authorizeDto) {
        List<Long> roleIds = RedisCache.getRoleIds(SaTokenUtils.getToken(), authorizeDto.getSystemId());
        return R.ok(roleIds);
    }

    @Override
    @ApiOperation("获取所有角色CODE")
    public R<List<String>> getRoleCodes(AuthorizeDto authorizeDto) {
        List<String> roleCodes = RedisCache.getRoleCodes(SaTokenUtils.getToken(), authorizeDto.getSystemId());
        return R.ok(roleCodes);
    }

    @Override
    @ApiOperation("是否是管理员")
    public R<Boolean> isAdmin(AuthorizeDto authorizeDto) {
        Boolean admin = RedisCache.isAdmin(SaTokenUtils.getToken(), authorizeDto.getSystemId());
        return R.ok(admin);
    }

    @Override
    @ApiOperation("获取所有菜单接口uri")
    public R<List<String>> getMenuInterfaceUri(AuthorizeDto authorizeDto) {
        List<String> menuInterfaceUri = RedisCache.getMenuInterfaceUri(SaTokenUtils.getToken(), authorizeDto.getSystemId());
        return R.ok(menuInterfaceUri);
    }

    @Override
    @ApiOperation("登录鉴权")
    public R<Boolean> authByLogin() {
        boolean b = RedisCache.authByLogin(SaTokenUtils.getToken());
        return R.ok(b);
    }

    @Override
    @ApiOperation("接口鉴权")
    public R<Boolean> authByInterface(AuthorizeDto authorizeDto) {
        boolean b = RedisCache.authByInterface(SaTokenUtils.getToken(), authorizeDto.getSystemId(), authorizeDto.getUri());
        return R.ok(b);
    }

    @Override
    @ApiOperation("数据鉴权")
    public R<Boolean> authByData(AuthorizeDto authorizeDto) {
        boolean b = RedisCache.authByData(SaTokenUtils.getToken(), authorizeDto.getSystemId(), authorizeDto.getUri());
        return R.ok(b);
    }

    @Override
    @ApiOperation("获取登录id")
    public R<Object> getLoginId() {
        Object loginId = RedisCache.getLoginId(SaTokenUtils.getToken(), Object.class);
        return R.ok(loginId);
    }

    @Override
    @ApiOperation("token续签")
    public R<Boolean> renewTimeout() {
        StpUtil.renewTimeout(SaTokenUtils.getToken(), saTokenAuthorizeConfig.getTimeout());
        return null;
    }
}
